flake: {config, pkgs, lib, ...}:

let
  cfg = config.services.mxmonzo;
  inherit (flake.packages.${pkgs.stdenv.hostPlatform.system}) mxmonzo;
in

with lib;

{
  options = {
    services.mxmonzo = {
      enable = mkOption {
        default = false;
        type = with types; bool;
        description = ''
          Start the Matrix Monzo bot.
        '';
      };

      user = mkOption {
        default = "mxmonzo";
        type = with types; uniq str;
        description = ''
          Name of the user.
        '';
      };

      matrixId = mkOption {
        type = with types; str;
        example = "@monzobot:librepush.net";
        description = ''
          Matrix ID of the Monzo bot user.
        '';
      };

      bindAddress = mkOption {
        default = "127.0.0.1:38320";
        type = with types; str;
        description = ''
          Host:Port upon which to bind the web interface (used for OAuth + webhooks).
        '';
      };

      externalBaseUri = mkOption {
        example = "https://mxmonzo.my.librepush.net";
        type = with types; str;
        description = ''
          External URL prefix to which this MxMonzo instance can be accessed (by webhooks).
          It should be proxied to the HTTP interface listening on `bindAddress`.
        '';
      };

      environmentFile = mkOption {
        type = with types; path;
        description = ''
          File containing environment variables, especially:
            - MATRIX_PASSWORD
            - MONZO_CLIENT_ID
            - MONZO_CLIENT_SECRET
        '';
      };

      matrixRoom = mkOption {
        type = with types; str;
        description = ''
          Room ID, like !roomid:librepush.net, of the bot's room to answer commands and emit notifications to.
        '';
      };

      dataPath = mkOption {
        type = with types; path;
        description = ''
          Path to where data can be kept.
        '';
      };
    };
  };

  config = mkIf cfg.enable {
    users.users."${cfg.user}" = {
      description = "Matrix Monzo User";
      isSystemUser = true;
      group = "${cfg.user}";
    };
    users.groups."${cfg.user}" = {};

    systemd.services.mxmonzo = {
      wantedBy = [ "multi-user.target" ];
      after = [ "network.target" ];
      description = "Start the Matrix Monzo bot.";

      environment = {
        BIND_ADDRESS = cfg.bindAddress;
        MATRIX_ID = cfg.matrixId;
        MATRIX_ROOM = cfg.matrixRoom;
        MATRIX_STORE = "${cfg.dataPath}/matrix-sdk";
        MATRIX_PERSIST = "${cfg.dataPath}/matrix.json";
        MONZO_PERSIST = "${cfg.dataPath}/monzo.json";
        BASE_URI = cfg.externalBaseUri;
      };

      serviceConfig = {
        Type = "simple";
        User = "${cfg.user}";
        ExecStart = ''${mxmonzo}/bin/mxmonzo'';

        EnvironmentFile = [
          cfg.environmentFile
        ];
      };
    };
  };
}
